"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GridImageJoiner = void 0;
const sharp_1 = __importDefault(require("sharp"));
const Loader_1 = require("./Loader");
const utils_1 = require("./utils");
class GridImageJoiner {
    images = [];
    grid_rows = 1;
    grid_cols = 1;
    loader = new Loader_1.FileLoader();
    constructor(grid_rows, grid_cols) {
        this.grid_rows = grid_rows;
        this.grid_cols = grid_cols;
    }
    sizeCheck(options) {
        const { row, col } = options;
        if (row >= this.grid_rows || col >= this.grid_cols) {
            throw new Error("row or col is out of range");
        }
    }
    pushImage(image, path) {
        (0, utils_1.assertExist)(image, path);
        this.sizeCheck(image.option);
        const { row_space = 1, col_space = 1 } = image.option;
        image.option = {
            ...image.option,
            row_space,
            col_space,
        };
        this.images.push(image);
    }
    loadImageFromLocal(path, options) {
        const image = this.loader.loadFileAsync(path);
        this.pushImage({ image, option: options });
    }
    loadImageFromObj(image, options) {
        this.pushImage({ image, option: options });
    }
    async getImageSize(image) {
        const image_ctx = (0, sharp_1.default)(image, { limitInputPixels: false });
        const meta_data = await image_ctx.metadata();
        return {
            width: meta_data.width,
            height: meta_data.height,
        };
    }
    async draw(options = {}) {
        if (this.images.length === 0) {
            throw new Error("No images to draw.");
        }
        const { resize, row_gap = 0, col_gap = 0, background_color = { r: 0, g: 0, b: 0, alpha: 0 }, } = options;
        let image_width, image_height;
        if (resize != null) {
            image_height = resize.height;
            image_width = resize.width;
        }
        else {
            const { height, width } = await this.getImageSize(await this.images[0].image);
            image_height = height;
            image_width = width;
        }
        const container_width = image_width * this.grid_cols + col_gap * (this.grid_cols - 1);
        const container_height = image_height * this.grid_rows + row_gap * (this.grid_rows - 1);
        const sharp_instance = await (0, sharp_1.default)({
            create: {
                width: container_width,
                height: container_height,
                channels: 4,
                background: background_color,
            },
            limitInputPixels: false,
            unlimited: true,
        });
        const ctx = sharp_instance.clone();
        const all_composite = [];
        const buffers = this.images.map(async (image_item) => {
            const { image, option } = image_item;
            const { row, col, row_space, col_space } = option;
            const input_image = await (0, sharp_1.default)(await image, { limitInputPixels: false })
                .resize(image_width * col_space, image_height * row_space)
                .toBuffer();
            return {
                input: input_image,
                top: row * (image_height + row_gap),
                left: col * (image_width + col_gap),
            };
        });
        ctx.composite(await Promise.all(buffers));
        await Promise.all(all_composite);
        return ctx;
    }
    clear() {
        this.images.length = 0;
    }
}
exports.GridImageJoiner = GridImageJoiner;
